const linkWrapperPrefBranch = "link_wrapper.";

var link_wrapper = {

  init: function() {
    link_wrapper.resetValues();

    var appcontent = document.getElementById("appcontent");   // browser

    if(appcontent) {
      appcontent.addEventListener("DOMContentLoaded", link_wrapper.linkParser, true);
    }
  },

  linkParser: function(e) {
    var ndDocument = e.originalTarget;
    var aBodyTags = ndDocument.getElementsByTagName("body");
    var ndBody = (aBodyTags.length > 0) ? aBodyTags[0] : false;
    
    var win = e.explicitOriginalTarget.defaultView
    var sIgnoreList = nsPreferences.copyUnicharPref(linkWrapperPrefBranch +"ignoreList");

    if (win.location && win.location.hostname && sIgnoreList) {
      var regEx_IgnoreList = new RegExp(sIgnoreList, "ig")
      try {
        if (regEx_IgnoreList.test(win.location.hostname)) {
          return false;
        }
      } catch(ex) {
        to_console(ex);
        return false;
      }
    }

    try {
      if (nsPreferences.getBoolPref(linkWrapperPrefBranch + "charLimitEnabled")) {
        var objRange = document.createRange();
        objRange.setEnd(ndBody, ndBody.childNodes.length);
        objRange.setStart(ndBody, 0);
  
        var nPageLength = objRange.toString().length;
        objRange.detach();
  
        if (nPageLength > nsPreferences.getIntPref(linkWrapperPrefBranch + "charLimit")) {
          return false;
        }
      }
    } catch (ex) {
      to_console(ex);
      return false;
    }

    var bModFlag = false;
    var winDoc = win.document;
    var regEx_IgnoreTags = new RegExp(nsPreferences.copyUnicharPref(linkWrapperPrefBranch + "ignoreTags"), "ig")

    var wrapLength = nsPreferences.getIntPref(linkWrapperPrefBranch + "wrapLength");
    var regEx_Wrapper = "\\S{" + wrapLength + "}";

    function line_parser(n) {
      var u, r, c, x;

      if (n.nodeType == 3) {
        u = n.data.search(regEx_Wrapper);
//          to_console(n.data + " : " + n.nodeType);

        if (u >= 0) {
          r = n.splitText(u + wrapLength);
          n.parentNode.insertBefore(winDoc.createElement('wbr'), r);
          bModFlag = true;
        }
      } else if (!regEx_IgnoreTags.test(n.tagName)) {
        for (c = 0; x = n.childNodes[c]; ++c) {
//          to_console(n.tagName);
          line_parser(x);
        }
      }
    }

    if (winDoc.body) {
      line_parser(winDoc.body);
    }

    if (bModFlag) {
      try {
        var headElem = window.content.document.getElementsByTagName("head")[0];
        if (!headElem) {
          return false;
        }

        elem = window.content.document.createElement("style");
        elem.id = "toggleprewrap";
        elem.type = "text/css";
        elem.innerHTML = "pre { white-space: -moz-pre-wrap !important; } /* inserted by the Link Wrapper extension */";
        headElem.appendChild(elem);

        // Coded added to fix regression is Firefox 2.0/3.0 dev builds
        var thisE = winDoc.body
        var p = thisE.parentNode;
        var n = thisE.nextSibling;
        p.removeChild(thisE);
        p.insertBefore(thisE, n);
    } catch (ex) {
        to_console(ex);
      }
    }

    return true;

  },
  
  getPrefBranch: function() {
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].
    getService(Components.interfaces.nsIPrefService);

    return prefService.getBranch(linkWrapperPrefBranch);
  },

  resetValues: function() {

    var myResetVersion = nsPreferences.copyUnicharPref(linkWrapperPrefBranch + "resetVersion");
    var thisResetVersion = "2.0.0.0";
    
    if (myResetVersion != thisResetVersion) {
      var nsIPrefBranchObj = link_wrapper.getPrefBranch();
      var prefCount = {value:0}; 
      var prefArray = nsIPrefBranchObj.getChildList("" , prefCount); 
  
      var branchService = nsPreferences.mPrefService;
  
      for (var i = 0; i < prefArray.length; ++i) {
        if (branchService.prefHasUserValue(linkWrapperPrefBranch + prefArray[i])) {
           branchService.clearUserPref(linkWrapperPrefBranch + prefArray[i]);
        }
      } 

      nsPreferences.setUnicharPref(linkWrapperPrefBranch + "resetVersion", thisResetVersion);
    }
  },
  
  unregister: function() {
    var appcontent = document.getElementById("appcontent");   // browser

    if (appcontent) {
      appcontent.removeEventListener("DOMContentLoaded", link_wrapper.linkParser, true);
      //appcontent.addEventListener("DOMTitleChanged", link_wrapper.resetOnPageChange, false);
    }
  },  
}

/* used for debugging only */
var mConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService)

function to_console(msg) {
  mConsoleService.logStringMessage("link_wrapper: " + msg);
}


//window.document.getElementById("appcontent").addEventListener("DOMContentLoaded", link_wrapper.linkParser, false);
// from http://kb.mozillazine.org/On_page_load
window.addEventListener("load", link_wrapper.init, false);
window.addEventListener("unload", link_wrapper.unregister, false);
